<?php
/*
Plugin Name: Foursquare Venue
Plugin URI: http://sutherlandboswell.com/2010/11/foursquare-venue-wordpress-plugin/
Description: Display your venue's foursquare stats in an easy-to-use widget or with the <code>[venue id=3945]</code> shortcode.
Author: Sutherland Boswell
Author URI: http://sutherlandboswell.com
Version: 3.0.1
License: GPL2
*/
/*  Copyright 2014 Sutherland Boswell  (email : sutherland.boswell@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as 
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// Define
define( 'FOURSQUARE_VENUE_PATH', dirname(__FILE__) );
define( 'FOURSQUARE_VENUE_VERSION', '3.0.1' );

// Includes
require_once( FOURSQUARE_VENUE_PATH . '/php/class-refactored-settings.php' );
require_once( FOURSQUARE_VENUE_PATH . '/php/widget.php' );

// Plugin Class
class Foursquare_Venue_Plugin {
	
	function __construct() {
		/**
		 * Settings
		 */
		$settings_args = array(
			'file'    => __FILE__,
			'version' => FOURSQUARE_VENUE_VERSION,
			'name'    => 'Foursquare Venue',
			'slug'    => 'foursquare_venue',
			'options' => array(
				'api_key' => array(
					'name'        => 'API Key',
					'description' => '<a href="https://foursquare.com/developers/apps">Register your app</a> and enter your keys below.',
					'fields'      => array(
						'client_id' => array(
							'name'        => 'Client ID',
							'type'        => 'text',
							'default'     => '',
							'description' => ''
						),
						'client_secret' => array(
							'name'        => 'Client Secret',
							'type'        => 'text',
							'default'     => '',
							'description' => ''
						)
					)
				),
				'display' => array(
					'name'        => 'Display',
					'description' => 'These settings control how the widget and <code>[venue]</code> shortcode are displayed.',
					'fields'      => array(
						'template' => array(
							'name'        => 'Template',
							'type'        => 'textarea',
							'class'       => 'code',
							'default'     => '<a href="%%foursquare_url%%">%%name%%</a><br>' . PHP_EOL . '%%checkins_count%% check-ins<br>' . PHP_EOL . 'Rating: %%rating%%<br>' . PHP_EOL . '%%here_now_summary%%',
							'description' => 'To view available placeholders, open the <a href="#" id="open-help-tab">Help</a> tab in the top right.<table class="foursquare-venue-template-preview-table"><tr><th>Preview</th></tr><tr><td><div id="foursquare-venue-template-preview"></div></td></tr></table>'
						)
					)
				),
			)
		);
		$this->settings = new Refactored_Settings_0_4( $settings_args );
		/**
		 * Add scripts and styles to settings page
		 */
		add_action( 'admin_init', array( &$this, 'settings_scripts' ), 20 );
		/**
		 * Add AJAX preview callback for settings page
		 */
		add_action( 'wp_ajax_foursquare_venue_template_preview', array( &$this, 'template_preview_callback' ) );
		/**
		 * Add Help tab
		 */
		add_action( 'load-settings_page_foursquare_venue', array( &$this, 'add_help_tab' ) );
		/**
		 * Register shortcode
		 */
		add_shortcode( 'venue', array( &$this, 'venue_shortcode' ) );
		/**
		 * Delete old settings
		 */
		$this->delete_old_settings();
	}

	/**
	 * Deletes pre-3.0 settings
	 */
	function delete_old_settings() {
		delete_option('foursquare_venue_client_id');
		delete_option('foursquare_venue_client_secret');
		delete_option('foursquare_venue_show_title');
		delete_option('foursquare_venue_stats_title');
		delete_option('foursquare_show_venue_name');
		delete_option('foursquare_show_venue_icon');
		delete_option('foursquare_show_here_now');
		delete_option('foursquare_show_here_now_text');
		delete_option('foursquare_show_total');
		delete_option('foursquare_show_total_text');
		delete_option('foursquare_show_mayor');
		delete_option('foursquare_show_mayor_text');
		delete_option('foursquare_link_mayor');
		delete_option('foursquare_show_mayor_photo');
		delete_option('foursquare_mayor_photo_size');
		delete_option('foursquare_venue_stats_width');
		delete_option('foursquare_venue_stats_align');
	}

	/**
	 * Adds javascript and CSS to the settings page
	 * @return void
	 */
	function settings_scripts() {
		wp_enqueue_script( 'foursquare-venue-settings-js', plugins_url( '/js/settings.js' , __FILE__ ), array( 'jquery' ), FOURSQUARE_VENUE_VERSION );
		wp_enqueue_style( 'foursquare-venue-admin-css', plugins_url( '/css/admin.css', __FILE__ ), false, FOURSQUARE_VENUE_VERSION );
	}

	/**
	 * The venue shortcode's callback function
	 * @param  array  $atts Shortcode attributes
	 * @return string       The shortcode's HTML
	 */
	function venue_shortcode( $atts ) {
		extract(shortcode_atts(array(
			'id' => '',
		), $atts));
		return $this->render_venue( $id );
	}

	/**
	 * Renders a venue's info as an HTML string
	 * @param  string $id The Foursquare venue ID
	 * @return string     An HTML string
	 */
	function render_venue( $id ) {
		$venue = $this->get_venue( $id );
		return $this->fill_template( $venue, $this->settings->options['display']['template'] );
	}

	/**
	 * Callback function for the AJAX call that displays a preview of the venue template
	 */
	function template_preview_callback() {
		// Security checks
		if ( !current_user_can( 'manage_options' ) ) die();

		$template = $_POST['template'];
		$venue = $this->get_venue( '3fd66200f964a520d6f01ee3' );

		echo $this->fill_template( $venue, $template );
		die();
	}

	/**
	 * Fills a template with venue data
	 * @param  object $venue    The venue data retrieved from Foursquare
	 * @param  string $template A template containing placeholders to be filled
	 * @return string           The HTML containing filled in data
	 */
	function fill_template( $venue, $template ) {
		if ( is_wp_error( $venue ) ) {
			$result = $venue->get_error_message();
		} else {
			$result = $template;
			$replacements = $this->get_replacements( $venue );
			foreach ( $replacements as $key => $value) {
				$result = str_replace( '%%' . $key . '%%', $value, $result );
			}
		}
		return $result;
	}

	/**
	 * Gets an array of values available for use in venue templates
	 * @param  object $venue The venue data retrieved from foursquare
	 * @return array         An array of keys available for replacement with real data as the value
	 */
	function get_replacements( $venue ) {
		$replacements = array(
			'id'               => $venue->id,
			'name'             => $venue->name,
			'phone'            => $venue->contact->phone,
			'formatted_phone'  => $venue->contact->formattedPhone,
			'twitter'          => $venue->contact->twitter,
			'address'          => $venue->location->address,
			'cross_street'     => $venue->location->crossStreet,
			'lat'              => $venue->location->lat,
			'lng'              => $venue->location->lng,
			'postal_code'      => $venue->location->postalCode,
			'cc'               => $venue->location->cc,
			'city'             => $venue->location->city,
			'state'            => $venue->location->state,
			'country'          => $venue->location->country,
			'foursquare_url'   => $venue->canonicalUrl,
			'short_url'        => $venue->shortUrl,
			'website_url'      => $venue->url,
			'checkins_count'   => $venue->stats->checkinsCount,
			'users_count'      => $venue->stats->usersCount,
			'tip_count'        => $venue->stats->tipCount,
			'likes_count'      => $venue->likes->count,
			'likes_summary'    => $venue->likes->summary,
			'rating'           => $venue->rating,
			'rating_signals'   => $venue->ratingSignals,
			'here_now_count'   => $venue->hereNow->count,
			'here_now_summary' => $venue->hereNow->summary,
			'description'      => $venue->description,
			'mayor_count'      => $venue->mayor->count,
			'mayor_id'         => $venue->mayor->user->id,
			'mayor_url'        => 'https://foursquare.com/user/' . $venue->mayor->user->id,
			'mayor_first_name' => $venue->mayor->user->firstName,
			'mayor_last_name'  => $venue->mayor->user->lastName,
			'mayor_gender'     => $venue->mayor->user->gender,
			'mayor_photo'      => $venue->mayor->user->photo->prefix . '100x100'. $venue->mayor->user->photo->suffix
		);
		return apply_filters( 'foursquare_venue/replacements', $replacements, $venue );
	}

	/**
	 * Get a venue's data from the Foursquare API or local cache
	 * @param  string $id The Foursquare venue ID
	 * @return object     The venue data
	 */
	function get_venue( $id ) {
		// Check for cached data
		$transient_name = 'foursquare_venue_' . $id;
		$data = get_transient( $transient_name );
		// If we don't have data, make an API request
		if ( $data === false ) {
			$request_url = 'https://api.foursquare.com/v2/venues/' . $id . '?client_id=' . $this->settings->options['api_key']['client_id'] . '&client_secret=' . $this->settings->options['api_key']['client_secret'] . '&v=20140702';
			$response = wp_remote_get( $request_url, array( 'sslverify' => false ) );
			if( is_wp_error( $response ) ) {
				$data = $response;
			} else {
				$data = json_decode( $response['body'] );
				if ( $data->meta->code != 200 ) {
					$data = new WP_Error( 'foursquare_api_error', $data->meta->errorDetail );
				} else {
					$data = $data->response->venue;
					set_transient( $transient_name, $data, 15 * 60 );
				}
			}
		}
		// Give up the data
		return $data;
	}

	/**
	 * This callback adds the help tab's content to the settings page
	 */
	function add_help_tab() {
		$screen = get_current_screen();

		$venue = $this->get_venue( '3fd66200f964a520d6f01ee3' );
		if ( is_wp_error( $venue ) ) {
			$table = '<p><strong>' . __( 'Error:' ) . '</strong> ' . $venue->get_error_message() . '</p>';
		} else {
			$table = '<table class="foursquare-venue-replacements"><thead><tr><th>' . __( 'Placeholder' ) . '</th><td>' . __( 'Example Value' ) . '</td></tr></thead><tbody>';
			$replacements = $this->get_replacements( $venue );
			foreach ( $replacements as $key => $value ) {
				$table .= '<tr><th>%%' . $key . '%%</th><td>' . $value . '</td></tr>';
			}
			$table .= '</tbody></table>';
		}

		// Add placeholder help tab
		$screen->add_help_tab( array(
			'id'	=> 'foursquare_venue_placeholder_help_tab',
			'title'	=> __('Venue Placeholders'),
			'content'	=> '<h2>' . __( 'Venue Placeholders' ) . '</h2><p>' . __( 'These placeholders will be replaced with the appropriate data in your display template.' ) . '</p>' . $table,
		) );
		// Add shortcode help tab
		$screen->add_help_tab( array(
			'id'	=> 'foursquare_venue_shortcode_help_tab',
			'title'	=> __('Venue Shortcode'),
			'content'	=> '<h2>' . __( 'Venue Shortcode' ) . '</h2><p>' . __( 'The <code>[venue]</code> shortcode can be used to display venue information and stats from Foursquare.' ) . '</p><p>' . __( 'To use the shortcode you will need the venue\'s Foursquare ID. The ID is found at the end of the venue\'s Foursquare URL. For example, The White House\'s venue URL is <code>https://foursquare.com/v/the-white-house/3fd66200f964a520d6f01ee3</code>, which means the ID is <code>3fd66200f964a520d6f01ee3</code>.' ) . '</p>' . __( 'Once you have the venue ID, you can insert the shortcode with the venue ID included. Example: <code>[venue id="3fd66200f964a520d6f01ee3"]</code>' ) . '</p>'
		) );

		$screen->set_help_sidebar( '<p>' . __( 'For support, please visit the <a href="http://wordpress.org/support/plugin/foursquare-venue">WordPress forums</a>.' ) . '</p>' );
	}

}

$foursquare_venue_plugin = new Foursquare_Venue_Plugin();

?>